import Vue from 'vue'

import Cookies from 'js-cookie'

import 'normalize.css/normalize.css' // A modern alternative to CSS resets

import Element from 'element-ui'
import 'element-ui/lib/theme-chalk/index.css'

import '@/styles/index.scss' // global css

import App from './App'
import router from './router'
import store from './store'
import global from './global.js'
import i18n from './lang' // Internationalization
import './icons' // icon
import './permission' // permission control

import * as filters from './filters' // global filters
import permission from '@/directive/permission/index.js' // 权限判断指令
import { getDicts, selectDictLabel, parseTime } from '@/api/system/dictData'
import {  download } from '@/utils/util'
Vue.use(Element, {
  size: Cookies.get('size') || 'medium', // set element-ui default size
  i18n: (key, value) => i18n.t(key, value)
})
Vue.prototype.download = download

Vue.prototype.$connectSocket= global.whetherWebSocket()
Vue.directive('permission', permission)
Vue.prototype.NumberMul = function(arg1, arg2) {
  var m = 0
  var s1 = arg1.toString()
  var s2 = arg2.toString()
  try {
    m += s1.split('.')[1].length
  } catch (e) {}
  try {
    m += s2.split('.')[1].length
  } catch (e) {}

  return Number(s1.replace('.', '')) * Number(s2.replace('.', '')) / Math.pow(10, m)
}
Vue.prototype.NumberAdd = function(arg1, arg2) {
  var r1, r2, m, n
  try {
    r1 = arg1.toString().split('.')[1].length
  } catch (e) {
    r1 = 0
  }
  try {
    r2 = arg2.toString().split('.')[1].length
  } catch (e) {
    r2 = 0
  }
  m = Math.pow(10, Math.max(r1, r2))
  n = (r1 >= r2) ? r1 : r2
  return ((arg1 * m + arg2 * m) / m).toFixed(n)
}
Vue.prototype.NumberSub = function(arg1, arg2) {
  var re1, re2, m, n
  try {
    re1 = arg1.toString().split('.')[1].length
  } catch (e) {
    re1 = 0
  }
  try {
    re2 = arg2.toString().split('.')[1].length
  } catch (e) {
    re2 = 0
  }
  m = Math.pow(10, Math.max(re1, re2))
  n = (re1 >= re2) ? re1 : re2
  return ((arg1 * m - arg2 * m) / m).toFixed(n)
}
Vue.prototype.NumberDiv = function (arg1, arg2, digit) {
  var t1=0, t2=0, r1, r2
  try { t1=arg1.toString().split('.')[1].length } catch (e) {}
  try { t2=arg2.toString().split('.')[1].length } catch (e) {}
  r1=Number(arg1.toString().replace('.', ''))
  r2=Number(arg2.toString().replace('.', ''))
  // 获取小数点后的计算值
  var result= ((r1/r2)*Math.pow(10, t2-t1)).toString()
  var result2=result.split('.')[1]
  result2=result2.substring(0, digit>result2.length?result2.length:digit)

  return Number(result.split('.')[0]+'.'+result2)
}
// register global utility filters.
Object.keys(filters).forEach(key => {
  Vue.filter(key, filters[key])
})
// 全局方法挂载
Vue.prototype.getDicts = getDicts
Vue.prototype.selectDictLabel = selectDictLabel
Vue.prototype.parseTime = parseTime
Vue.config.productionTip = false
Vue.prototype.$global = global
new Vue({
  el: '#app',
  router,
  store,
  i18n,
  render: h => h(App)
})
